using System;
using System.IO;
using System.Diagnostics;

namespace LazyCaching
{
	/// <summary>
	/// This can be any cachable business object
	/// </summary>
	public class ConcreteCacheable : ICached, ICacheable 
	{
		// declare expensive object
		byte[] m_baConfigData;

		public ConcreteCacheable()
		{
			// perform the usual initialization
		}

		/// <summary>
		/// Property for our configuration data
		/// </summary>
		public byte[] ConfigData
		{
			get { return m_baConfigData; }
			set { m_baConfigData = value; }
		}

		/// <summary>
		/// Factory Method for both object creation and 
		/// determining caching semantics (e.g. setting cache or getting already cached object)
		/// Our CacheAdapter is used to avoid coupling our busienss objects with any particular cache
		/// </summary>
		/// <param name="oCache"></param>
		/// <param name="sKey"></param>
		/// <returns></returns>
		public static ICached PrepareCache(CacheAdapter oCache, string sKey)
		{
			ICached oCachedObject = null;

			// if using the cache object check to see if this object is already there
			// any logic could be used here to refresh the cache as well instead of merely checking if not object existing int the cache
			if (oCache[sKey] == null)
			{
				Trace.WriteLine("PrepareCache: Cache object inserted into cache\n" + 
					"Current Cache Key Used = " + sKey + "\n");
							
				// prepare expensive config buffer
				ConcreteCacheable oCacheable = new ConcreteCacheable();
				oCacheable.PrepareConfigData();
				
				// add any other expensive operation here....
				// ...
		
				// now insert myself into the passed cache object
				// this is where the business object determines expiration etc....
				oCache.Insert(sKey, oCacheable, DateTime.MaxValue, TimeSpan.FromMinutes(60));
		
			}
		
			Trace.Write("Execute: Retrieving Item from Cache\n" + 
					"Cache Object Retrieved (By Key) = " + sKey + "\n");
			
			// just return existing cachable object's interface for cached execution....
			oCachedObject = (ICached)oCache[sKey];
			
			return oCachedObject;
		}

		/// <summary>
		/// This can be an expensive operation, db i/o, file i/o, etc.
		/// </summary>
		public void PrepareConfigData()
		{
			FileStream oFileStream = null;
			int nFileByteLength = 0;

			// create and set expensive objects here
			// open up our config file and read in the contents
			oFileStream = new FileStream("c:\\config.xml", FileMode.OpenOrCreate, FileAccess.ReadWrite);
			nFileByteLength = (int) oFileStream.Length;
			ConfigData = new byte[nFileByteLength];
			// save contents into config buffer
			oFileStream.Read(ConfigData, 0, nFileByteLength);
			
			// close the file
			oFileStream.Close();
		}

		/// <summary>
		/// This can be an expensive operation, db i/o, file i/o, etc.
		/// </summary>
		public void PrepareWorkers()
		{
			// add expensive object initializations here e.g. unmanaged proxy's etc.
		}

		public void ExecuteCached()
		{
			// read in configuration data
			if (ConfigData == null)
				PrepareConfigData();

			// use config data for something....
		}

		public void ExecuteWithoutCache()
		{
			// read in configuration data
			if (ConfigData == null)
				PrepareConfigData();

			// use config data for something....
		}
	}
}
