using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Web;
using System.Web.Services;
using DotNetPatterns.Common;
using DataTierPatterns.Data;
using System.Collections.Specialized;

namespace DataServiceFacade
{
	/// <summary>
	/// Driver facade for all Data Tier Patterns
	/// </summary>
	public class DataServiceFacade : System.Web.Services.WebService
	{
		public DataServiceFacade()
		{
			//CODEGEN: This call is required by the ASP.NET Web Services Designer
			InitializeComponent();
		}

		#region Component Designer generated code
		
		//Required by the Web Services Designer 
		private IContainer components = null;
				
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if(disposing && components != null)
			{
				components.Dispose();
			}
			base.Dispose(disposing);		
		}
		
		#endregion

		[WebMethod]
		public DataSet GenerateCustomerSchema(string sProduct, string sVersion, string sDescription, bool bStore)
		{
			Customers dsCustomers = null;
			try
			{
				dsCustomers = new Customers();
	
				// add lookup instance data at this time...
				Customers.LOOKUPRow oRow = dsCustomers.LOOKUP.NewLOOKUPRow();

				// solely for testing and showing instance data....
				oRow.COLUMN = "CompanyName";
				oRow.TABLE = "Customer";
				dsCustomers.LOOKUP.AddLOOKUPRow(oRow);
				dsCustomers.LOOKUP.AcceptChanges();

				oRow = dsCustomers.LOOKUP.NewLOOKUPRow();
				oRow.COLUMN = "ContactName";
				oRow.TABLE = "Customer";
				dsCustomers.LOOKUP.AddLOOKUPRow(oRow);
				dsCustomers.LOOKUP.AcceptChanges();
				
				if (bStore)
					StoreSchema(sProduct, sVersion, sDescription, dsCustomers);
			}
			catch(Exception ex)
			{
				throw new BaseException(Utilities.BuildErrorMessage("Exception thrown", ex.Message), true);
			}
			return dsCustomers;
		}

		[WebMethod]
		public DataSet GenerateLookupSchema(string sProduct, string sVersion, string sDescription, bool bStore)
		{
			Customers dsCustomers = null;
			try
			{
				Customers dsCustomerLookup = new Customers();
					
				Customers.LOOKUPRow oRow = dsCustomerLookup.LOOKUP.NewLOOKUPRow();

				// solely for testing and showing instance data....
				oRow.COLUMN = "CompanyName";
				oRow.TABLE = "Customers";
				dsCustomerLookup.LOOKUP.AddLOOKUPRow(oRow);
				dsCustomerLookup.LOOKUP.AcceptChanges();

				oRow.COLUMN = "ContactName";
				oRow.TABLE = "Customers";
				dsCustomerLookup.LOOKUP.AddLOOKUPRow(oRow);
				dsCustomerLookup.LOOKUP.AcceptChanges();
				
				if (bStore)
					StoreSchema(sProduct, sVersion, sDescription, dsCustomers);
			}
			catch(Exception ex)
			{
				throw new BaseException(Utilities.BuildErrorMessage("Exception thrown", ex.Message), true);
			}
			return dsCustomers;
		}

		[WebMethod]
		public DataSet GenerateSampleCustomerData()
		{
			Customers dsCustomers = null;
			try
			{
				dsCustomers = new Customers();
	
				Customers.CustomerRow oRow = dsCustomers.Customer.NewCustomerRow();

				// solely for testing and showing instance data....
				oRow.CustomerID = "123";
				oRow.CompanyName = "The eTier Group";
				oRow.ContactName = "Christian Thilmany";
				oRow.Phone = "713-555-2343";
				dsCustomers.Customer.AddCustomerRow(oRow);
				dsCustomers.Customer.AcceptChanges();

				long lTransID = GenerateTransactionID();

				Packet oPacket = new Packet();
				oPacket.TransactionID = lTransID;
				oPacket.RawData = dsCustomers;
				oPacket.Data = dsCustomers.Tables;
				oPacket.ProductSchemaID = 100018; // Can optionally make call to get latest schema
				oPacket.ProductIdentitySchemaID = 100018; // see above comment

				ModelFactory.CreatePolyModel().StoreTransaction(oPacket);

			}
			catch(Exception ex)
			{
				throw new BaseException(Utilities.BuildErrorMessage("Exception thrown", ex.Message), true);
			}
			return dsCustomers;
		}

		[WebMethod]
		public DataSet GenerateSampleCustomerLookupData()
		{
			Customers dsCustomerLookup = null;
			try
			{
				dsCustomerLookup = new Customers();
					
				Customers.LOOKUPRow oRow = dsCustomerLookup.LOOKUP.NewLOOKUPRow();

				oRow = dsCustomerLookup.LOOKUP.NewLOOKUPRow();

				oRow.COLUMN = "ContactName";
				oRow.TABLE = "Customer";
				dsCustomerLookup.LOOKUP.AddLOOKUPRow(oRow);
				dsCustomerLookup.LOOKUP.AcceptChanges();
				
				long lTransID = GenerateTransactionID();

				Packet oPacket = new Packet();
				oPacket.TransactionID = lTransID;
				oPacket.RawData = dsCustomerLookup;
				oPacket.ProductSchemaID = 100016;
				
				ModelFactory.CreatePolyModel().StoreTransaction(oPacket);

			}
			catch(Exception ex)
			{
				throw new BaseException(Utilities.BuildErrorMessage("Exception thrown", ex.Message), true);
			}
			return dsCustomerLookup;
		}

		[WebMethod]
		public DataSet StoreSchema(string sProduct, string sVersion, string sDescription, DataSet ds)
		{
			try
			{
				ModelFactory.CreatePolyModel().StoreSchema(sProduct, sVersion, DTPConstants.SCHEMA_TYPE.PRODUCT, sDescription, ds);
			}
			catch (Exception ex)
			{
				throw new BaseException(Utilities.BuildErrorMessage("Exception thrown", ex.Message), true);
			}
			
			return null;
		}

		[WebMethod]
		public DateTime GetCurrentDBDate()
		{
			return ModelFactory.CreatePolyModel().GetCurrentDBDate();
		}

		[WebMethod]
		public long GenerateTransactionID()
		{
			return ModelFactory.CreatePolyModel().GenerateTransactionID();			 			 			 
		}


		[WebMethod]
		public DataSet GetTransactionFromPacket(long lTransID)
		{
			Packet oPacket = new Packet();
			oPacket.TransactionID = lTransID;
			return ModelFactory.CreatePolyModel().GetTransaction(oPacket).RawData;
		}

		[WebMethod]
		public DataSet GetTransactionByWhereCollection()
		{
			PolyModelWhereCollection colWhere = ModelFactory.CreatePolyModelWhereCollection();
			colWhere.Add("CompanyName", "The eTier Group");

			return ModelFactory.CreatePolyModel().GetTransaction(colWhere).RawData;
		}

		[WebMethod]
		public DataSet GetTransactionByWhereCollectionEx(string sColumn1, string sValue1, string sColumn2, string sValue2, string sColumn3, string sValue3)
		{
			PolyModelWhereCollection colWhere = ModelFactory.CreatePolyModelWhereCollection();
			if (sColumn1 != string.Empty)
				colWhere.Add(sColumn1, sValue1);
			if (sColumn2 != string.Empty)
				colWhere.Add(sColumn2, sValue2);
			if (sColumn3 != string.Empty)
				colWhere.Add(sColumn3, sValue3);
			
			return ModelFactory.CreatePolyModel().GetTransaction(colWhere).RawData;
		}


		[WebMethod]
		public long GetTransactionsByTransIdArray(long lTransID1, long lTransID2, long lTransID3)
		{
			long[] arIDS = {lTransID1, lTransID2, lTransID3};			
			return ModelFactory.CreatePolyModel().GetTransactions(arIDS).Count;
		}

		[WebMethod]
		public int GetTransactionsBySchemaID(long lSchemaID)
		{
			return ModelFactory.CreatePolyModel().GetTransactions(lSchemaID).Count;
		}

		[WebMethod]
		public long GetCounterValue(string sCounterName)
		{
			return ModelFactory.CreatePolyModel().GetCounterValue(sCounterName);
		}
		

		[WebMethod]
		public long GetNextCounterValue(string sCounterName)
		{
			return ModelFactory.CreatePolyModel().GetNextCounterValue(sCounterName);
		}
		

		[WebMethod]
		public bool StoreCounterValue(string sCounterName, long lDefaultValue)
		{
			return ModelFactory.CreatePolyModel().StoreCounterValue(sCounterName, lDefaultValue);
		}

		[WebMethod]
		public DataSet GetTransactionByID(long lTransactionID)
		{
			return ModelFactory.CreatePolyModel().GetTransaction(lTransactionID).RawData;
		}

		[WebMethod]
		public bool DoesSchemaExist(string sSchemaName, string sVersion, DTPConstants.SCHEMA_TYPE eType)
		{
			return ModelFactory.CreatePolyModel().DoesSchemaExist(sSchemaName, sVersion, eType);
		}

		[WebMethod]
		public DataSet GetSchemaAsString(string sSchemaName, string sVersion, DTPConstants.SCHEMA_TYPE eType)
		{
			PolyModelData oDB = ModelFactory.CreatePolyModelData();
			oDB.ReadXmlSchema(ModelFactory.CreatePolyModel().GetSchemaAsString(sSchemaName, sVersion, eType));
			return oDB.Data;
		}	
		
		[WebMethod]
		public DataSet GetSchemaAsPacket(string sSchemaName, string sVersion, DTPConstants.SCHEMA_TYPE eType)
		{
			return ModelFactory.CreatePolyModel().GetSchema(sSchemaName, sVersion, eType).RawData;
		}


		[WebMethod]
		public DataSet GetSchemaByID(long lID)
		{
			return ModelFactory.CreatePolyModel().GetSchema(lID).RawData;
		}
		
	}
}
