using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Web;
using System.Web.Services;
using System.Text;
using DotNetPatterns.Common;
using System.Data.OleDb;
using System.Web.Services.Protocols;

namespace ExternalService
{
	/// <summary>
	/// Sample external web service that is used for testing...
	/// </summary>
	public class ExternalService : System.Web.Services.WebService
	{
		/// <summary>
		/// used for security credentials stuffed in a soap header
		/// </summary>
		public TransactorHeader oTransHeader;
		
		/// <summary>
		/// see oAuthHeader
		/// </summary>
		public class TransactorHeader : SoapHeader
		{
			public long lTransactorId;	
		}
		
		public ExternalService()
		{
			//CODEGEN: This call is required by the ASP.NET Web Services Designer
			InitializeComponent();
		}

		[WebMethod]
		public string GetTransaction()
		{
			return "here is your transaction";
		}

		#region Component Designer generated code
		
		//Required by the Web Services Designer 
		private IContainer components = null;
				
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if(disposing && components != null)
			{
				components.Dispose();
			}
			base.Dispose(disposing);		
		}
		
		#endregion

		/// <summary>
		/// This can be any external service, web service or method, this just exemplifies a long running
		/// transaction/action of some kind, it doesn't necessarily involve database persistence.
		/// </summary>
		[WebMethod]
		[SoapHeaderAttribute("oTransHeader", Direction=SoapHeaderDirection.In)]
		public DataSet InvokeTimeConsumingMethod()
		{
			DataSet ds = new DataSet();
			DataSet dsCopy = null;

			try
			{
				// something that will take up time ...
				for (int nCount = 0; nCount < 1000; nCount++)
				{
					// call any long running transaction or in this case a database op that take a bit to complete
					ds = GetAllCustomers();
			
					dsCopy = new DataSet();
					dsCopy = ds.Copy();
				}
			}
			catch (Exception ex)
			{
				throw new BaseException(Utilities.BuildErrorMessage("Exception thrown", ex.Message), true);
			}
			return ds;
		}

		/// <summary>
		/// Helper Method to retrive call customer from Northwind databse, used just for testing
		/// </summary>
		/// <returns></returns>
		public DataSet GetAllCustomers()
		{
			OleDbConnection cn = null;
			DataSet ds = null;
			OleDbDataAdapter da = null;

			try
			{
				cn = new OleDbConnection();
				
				#region Access Connection Properties
				// Connection string for using access
				//cn.ConnectionString =	"Provider=Microsoft.JET.OLEDB.4.0;" +
				//	@"data source = C:\Program Files\Microsoft Office\Office\Samples\Northwind.mdb";
				#endregion

				#region SQL Server Connection Properties
				// for SQL Server Test 
				cn.ConnectionString =	"Provider=SQLOLEDB;" +
					"Integrated Security=SSPI;" +
					"Persist Security Info=False;" + 
					"Initial Catalog=Northwind;" + 
					"Data Source=DEV06;";
				#endregion

				cn.Open();
				da = new OleDbDataAdapter("Select * from Customers", cn);
				ds = new DataSet("Customers");
				da.Fill(ds, "Customers");
			}
			catch (OleDbException dbEx)
			{
				throw new BaseException(Utilities.BuildErrorMessage("OleDbException thrown", dbEx.Message), true);
			}
			catch (Exception ex)
			{
				throw new BaseException(Utilities.BuildErrorMessage("Exception thrown", ex.Message), true);
			}
			finally
			{
				da.Dispose();
				cn.Close();
			}

			return ds;
		}
	}
}
