
#include "stdafx.h"
#include "ExternalService.h"

#include "LCTClient.h"

#define CLIENTS_COUNT 8

//##ModelId=3CFFC3BC00C5
struct stCallbackParam
{
	//##ModelId=3CFFC3BC00E4
	int		iServerIndex;
	//##ModelId=3CFFC3BC00E5
	LPLONG	lpCompletionCounter;
};

/*
//	This is the callback that is registered with the LCTClient class during PrepareAsync
*/
void LCTCallback(LPARAM lParam, HRESULT hRet)
{
	stCallbackParam	*param= reinterpret_cast<stCallbackParam*>(lParam);
	ATLASSERT(param);

	// increment the shared counter 
	InterlockedIncrement(param->lpCompletionCounter);

	// do something useful with the HRESULT
    printf("\n[Client %d] - Async Result : %s", param->iServerIndex, 
										SUCCEEDED(hRet)?"SUCCEEDED":"FAILED");
}

/*
// This just drives the LCT Client to execute GetTransaction from an external web service pooling up 
// several clients in the process.  The number of clients is configurable.
*/
int _tmain(int argc, _TCHAR* argv[])
{
	CLCTClientThreadPool	threadPool;
	stCallbackParam			paramsExternalService[CLIENTS_COUNT];
	BSTR	bstrRet;
	int		iIndex = 0;
	int		iPoolSize = CLIENTS_COUNT;
	HRESULT	hRet;
	LONG	lCompletionCounter = 0;
	int nCallCounter = 0;
		
	CoInitialize(NULL);
	
	// Initialize the tread pool to 4 threads per CPU
	threadPool.Initialize(0, -4);

	{
		// allocate an array of the number of pooled clients you want
		CExternalServiceT<CLCTClientSoapSocket> proxiesExternalService[CLIENTS_COUNT];

		// loop through each imaginary client, prepare request, and call asynchronously using LCT client for heavy lifting
		for(iIndex = 0; iIndex < iPoolSize; iIndex++)
		{
			// server index can be replace by some unique cookie/handle for async ops
			paramsExternalService[iIndex].iServerIndex = iIndex + 1;
			paramsExternalService[iIndex].lpCompletionCounter = &lCompletionCounter;
			
			// pass in an initialized threadpool (the number is up to you), the callback and the parameters
			hRet = proxiesExternalService[iIndex].PrepareRequest(&threadPool, 
									LCTCallback, (LPARAM)&paramsExternalService[iIndex]);
			ATLASSERT(SUCCEEDED(hRet));

			// just call the sample GetTransaction Web service method, the parm will be the return value OUT of that call
			hRet = proxiesExternalService[iIndex].GetTransaction(&bstrRet);
			
			// if E_PENDING things are working..
			ATLASSERT(E_PENDING == hRet);
		}

		// Request in work, yet
		do
		{
			Sleep(50);
			printf(".");
		}
		while(lCompletionCounter < CLIENTS_COUNT);
			
		// call callbacks have been called we can continue and recieve the operation return values
		for(iIndex=0; iIndex<CLIENTS_COUNT; iIndex++)
		{
			printf("\nClient %d : ", iIndex + 1);

			// Back from the request
			// Collect the results now
			// Pass only the OUT and IN/OUT params
			// The rest will be ignored
			hRet = proxiesExternalService[iIndex].GetTransaction(&bstrRet);
			
			// now really analyze the results
			if(SUCCEEDED(hRet))
			{
				nCallCounter++;
				printf("Requests Succeeded. Result :\n %ls", bstrRet);
				::SysFreeString(bstrRet);
			}
			else
				printf("Request Failed");
			
		}
		if(nCallCounter == CLIENTS_COUNT)
		{
			::MessageBox(NULL, _T("All async calls to the external service have completed."),_T("Status"),MB_OK);
		}
				

	}

	// make sure and cleanup the threadpool
	printf("\n");
	threadPool.Shutdown();
	CoUninitialize();

	return 0;
}

