using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Web;
using System.Web.Services;
using System.Text;
using DotNetPatterns.Common;

namespace LCTServer
{
	/// <summary>
	/// This is the driver web service for the loosely coupled transactor
	/// From this web service, another service can be called asynchronously
	/// and its status queried and eventually the transaction results returned
	/// </summary>
	public class TransactorService : System.Web.Services.WebService
	{
		public TransactorService()
		{
			//CODEGEN: This call is required by the ASP.NET Web Services Designer
			InitializeComponent();
		}

		#region Component Designer generated code
		
		//Required by the Web Services Designer 
		private IContainer components = null;
				
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if(disposing && components != null)
			{
				components.Dispose();
			}
			base.Dispose(disposing);		
		}
		
		#endregion

		/// <summary>
		/// Executes a particular external service asynchronously, can be easily made to
		/// call chained or unchained service factory (chapter 8) to launch any external service
		/// </summary>
		[WebMethod]
		public long Execute()
		{
			ITransactor oTransactor = (ITransactor) new TransactorFacade();
			long lTransactorId = 0;
			localhost.ExternalService oExtService = new localhost.ExternalService();
						
			try
			{
				// generate a unique transactor id 
				lTransactorId = oTransactor.GenerateTransactionId();

				// insert the job into the job table and give it a pending status STATUS = 1
				oTransactor.CreateTransaction(lTransactorId, 1);  // 1 for pending

				// simply delegate to the InvokeTimeConsumingMethod call... this can be any call including a chainedservicefactory
				oExtService.TransactorHeaderValue = new localhost.TransactorHeader();
				oExtService.TransactorHeaderValue.lTransactorId = lTransactorId;
				oExtService.BeginInvokeTimeConsumingMethod(new System.AsyncCallback(TransactorService.ExternalServiceCallback), oExtService);	
			}
			catch (Exception e)
			{
				throw new BaseException(Utilities.BuildErrorMessage("LCT Server Exception thrown", e.Message), true);
			}
			
			return lTransactorId;
		}

		/// <summary>
		/// Returns the just the status of the original transaction
		/// </summary>
		[WebMethod]
		public int GetStatus(long lTransactorId)
		{
			ITransactor oTransactor = (ITransactor) new TransactorFacade();
			int nStatus = 0;
			
			try
			{
				// insert the job into the job table and give it a pending status STATUS = 1
				nStatus = oTransactor.GetStatus(lTransactorId);

			}
			catch (Exception e)
			{
				throw new BaseException(Utilities.BuildErrorMessage("LCT Server Exception thrown", e.Message), true);
			}
			
			return nStatus;
		}

		/// <summary>
		/// Executes a particular external service asynchronously, can be easily made to
		/// call chained or unchained service factory (chapter 8) to launch any external service
		/// </summary>
		[WebMethod]
		public DataSet GetTransaction(long lTransactorId)
		{
			ITransactor oTransactor = (ITransactor) new TransactorFacade();
			DataSet ds = null;
			
			try
			{
				// insert the job into the job table and give it a pending status STATUS = 1
				ds = oTransactor.GetTransaction(lTransactorId); 

			}
			catch (Exception e)
			{
				throw new BaseException(Utilities.BuildErrorMessage("LCT Server Exception thrown", e.Message), true);
			}
			
			return ds;
		}

		/// <summary>
		///	Asynchronous Callback method for async ops
		/// </summary>
		/// <param name="ar"></param>
		public static void ExternalServiceCallback(IAsyncResult ar) 
		{
			DataSet dsOriginal = new DataSet();
			ITransactor oTransactor = (ITransactor) new TransactorFacade();
			
			try
			{
				// Obtains return value from the delegate call using EndInvoke.
				localhost.ExternalService oExtService = (localhost.ExternalService)ar.AsyncState;				
				dsOriginal = oExtService.EndInvokeTimeConsumingMethod(ar);

				// update the transaction with a success (==2) and the results of the original transaction
				if (dsOriginal != null)
					oTransactor.UpdateTransaction(oExtService.TransactorHeaderValue.lTransactorId, 2, dsOriginal);
				
			}
			catch(Exception e)
			{
				throw new BaseException(Utilities.BuildErrorMessage("LCTServer Exception thrown", e.Message), true);
			}
		}

	}
}
