using System;
using System.Runtime.InteropServices;

namespace ThreadDelegate
{
	internal class HighFrequencyTimer
	{
		[DllImport("kernel32.dll")]
		private static extern int QueryPerformanceFrequency(out long frequency);
	
		[DllImport("kernel32.dll")]
		private static extern int QueryPerformanceCounter(out long tick);
	
		private long _start;
		private long _stop;
		private long _timerFrequency;
	
		public HighFrequencyTimer()
		{
			QueryPerformanceFrequency(out _timerFrequency);

			if (_timerFrequency == 0) 
				throw new InvalidOperationException("Hardware does not support the high resolution timer.");
		}
	
		public long ElapsedTicks
		{
			get	{return (_stop - _start);}
		}
	
		public decimal ElapsedMicroseconds
		{
			get
			{
				return this.ElapsedSeconds * 1000 * 1000;
			}
		}
	
		public decimal ElapsedMilliseconds
		{
			get
			{
				return this.ElapsedSeconds * 1000;
			}
		}

		public decimal ElapsedSeconds
		{
			get
			{
				return ((decimal)this.ElapsedTicks / (decimal)_timerFrequency);
			}
		}
	
		public TimeSpan ElapsedTimeSpan
		{
			get
			{
				return new TimeSpan(this.ElapsedTicks);
			}
		}
	
		public long TicksPerSecond
		{
			get {return _timerFrequency;}
		}
	
		public void Start()
		{
			QueryPerformanceCounter(out _start);
		}
	
		public long Stop()
		{
			QueryPerformanceCounter(out _stop);
		
			return this.ElapsedTicks;
		}
	}
}
