using System;

namespace Templated
{
	/// <summary>
	/// Will indicate what mode a TemplateBase
	/// page should be shown in.
	/// </summary>
	public enum Mode : int
	{
		/// <summary>
		/// Render normally.
		/// </summary>
		Normal = 0,
		/// <summary>
		/// Render the error output
		/// </summary>
		Error = 1
	}
	
	/// <summary>
	/// Summary description for TemplateBase.
	/// </summary>
	public class TemplateBase : System.Web.UI.Page
	{

		public TemplateBase()
		{
		}

		#region Page Title
		private string mstrPageTitle = "[Sample]";
		public string PageTitle 
		{
			get { return mstrPageTitle; }
			set { mstrPageTitle = value; }
		}
		#endregion

		#region Page Mode
		private Mode mPageMode = Mode.Normal;
		/// <summary>
		/// Will allow you to retrieve the current page mode.
		/// To set the page mode to error, use the "SetPageError"
		/// method.
		/// </summary>
		public Mode PageMode
		{
			get { return mPageMode; }
		}
		#endregion

		#region Error Handling
		private string mPageErrorText = "";
		public virtual string PageErrorText
		{
			get { return mPageErrorText; }
		}
		private string mErrorXRef = "";
		public string ErrorXRef
		{
			get 
			{
				// We only want to retrieve this once.
				if (mErrorXRef.Length == 0)
					mErrorXRef = ErrorXRefGenerator.NextID();
				return mErrorXRef;
			}
		}

		protected void SetError(string Text, Exception ex)
		{
			try
			{
				mPageErrorText = Text;
				mPageMode = Mode.Error;
			}
			catch(Exception ex1)
			{
				System.Diagnostics.Debug.WriteLine(ex1.ToString());
			}

			string assemblyName = "";
			try
			{
				assemblyName = System.Reflection.Assembly.GetExecutingAssembly().GetName().Name;
				if (!System.Diagnostics.EventLog.SourceExists(assemblyName))
				{
					System.Diagnostics.EventLog.CreateEventSource(assemblyName, "");
				}
			}
			catch(Exception ex2)
			{
				System.Diagnostics.Debug.WriteLine(ex2.ToString());
			}

			try
			{
				System.Diagnostics.EventLog.WriteEntry(assemblyName, 
					String.Format("{0}\n{1}\n\n{2}", ErrorXRef, Text, ex.ToString()), 
					System.Diagnostics.EventLogEntryType.Error);
			}
			catch(Exception ex3)
			{
				System.Diagnostics.Debug.WriteLine(ex3.ToString());
			}
		}
		#endregion

		#region Template functions
		protected string TemplateTop()
		{
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			sb.Append("<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.0 Transitional//EN' >\n");
			sb.Append("<HTML>\n");
			sb.Append("	<HEAD>\n");
			sb.AppendFormat("		 <title>{0}</title>\n", mstrPageTitle);
			sb.Append("	</HEAD>\n");
			sb.Append("	<body>\n");
			sb.Append("<TABLE id=Table1 cellSpacing=1 cellPadding=1 width=\"100%\" height=100% border=0>\n");
			sb.Append("<TR valign=top><td>\n");
			sb.Append("<TABLE id=Table1 cellSpacing=1 cellPadding=1 width=\"100%\" border=0>\n");
			sb.Append("<TR valign=bottom>\n");
			sb.Append("	<TD rowspan=3 width=187><IMG \n");
			sb.Append("	src=\"logo.gif\" ></TD>\n");
			sb.Append("	<TD>\n");
			sb.Append("	<P align=center><FONT size=5>ANOTHER GENERIC WEBSITE</FONT></P></TD></TR>\n");
			sb.Append("<TR valign=top>\n");
			sb.Append("	<TD>\n");
			sb.Append("	<P align=center>Where you go to find nothing in \n");
			sb.Append("particular</P></TD></TR>\n");
			sb.Append("<TR valign=bottom>\n");
			sb.Append("	<TD align=right>\n");
			sb.AppendFormat("		Page: {0}</P></TD></TR></TABLE>\n", mstrPageTitle);
			sb.Append("\n\n      <!-- End of Start Page -->\n\n");
			return sb.ToString();

		}

		protected string TemplateBottom()
		{
			System.Text.StringBuilder sb = new System.Text.StringBuilder();

			sb.Append("\n\n      <!-- Start End of Page -->\n\n");
			sb.Append("  </td></tr>\n");
			sb.Append("  <tr valign=bottom><td>\n");
			sb.Append("			<P align='center'>\n");
			sb.Append("			  <a href='HelloWorld.aspx'>Hello World</a> | \n");
			sb.Append("			  <a href='HelloWorld2.aspx'>Super Hello World</a> | \n");
			sb.Append("			  <a href='Default.aspx'>Home</a> | \n");
			sb.Append("			  <a href='ExceptionPage.aspx'>Exception</a></P>\n");
			sb.Append("			<P align='center'>Copyright (c) 2003 Some Corp, Inc.</P>\n");
			sb.Append("  </td></tr></table>\n");
			sb.Append("	</body>\n");
			sb.Append("</HTML>\n");

			return sb.ToString();
		}

		protected string ShowErrorSection()
		{
			System.Text.StringBuilder sb = new System.Text.StringBuilder();

			sb.Append("\n\n      <!-- Start End of Page -->\n\n");
			sb.Append("  </td></tr>\n");
			sb.Append("  <tr valign=top bgcolor=whitesmoke><td>\n");
			sb.Append("    An error occurred while processing your request.<br>\n");
			
			string strPageErrorText = PageErrorText;
			if (strPageErrorText.Length > 0)
			{
				sb.AppendFormat("    <br>\n{0}<br>\n<br>\n", strPageErrorText);
			}

			sb.AppendFormat("    Please reference incident {0} when contacting tech support <br>\n<br>\n<small>Error generated {1}</small><br>\n", ErrorXRef, DateTime.Now);

			return sb.ToString();
		}

		#endregion 

		#region Send the page to the user
		protected override void Render(System.Web.UI.HtmlTextWriter writer)
		{
	
			// We do each write in a separate try/catch block to 
			// try to make sure we get as many of the writes as
			// possible.  It also allows us to make sure that any
			// exceptions thrown during the early writes will put
			// us into "Error" mode and include the standard 
			// exception mode.

			try
			{
				// Write out the top of the page.
				writer.Write(TemplateTop());
			}
			catch(Exception ex)
			{
				SetError("Error rendering page (AAA)", ex);
			}

			try
			{
				// Show the normal page mode if we are in
				// normal mode.  
				if (PageMode == Mode.Normal)
					base.Render(writer);
			}
			catch(Exception ex)
			{
				SetError("Error rendering page (AAB)", ex);
			}

			try
			{
				// Show the error page mode
				if (PageMode == Mode.Error)
					writer.Write(ShowErrorSection());
			}
			catch(Exception ex)
			{
				SetError("Error rendering page (AAC)", ex);
			}

			try
			{
				// Write out the bottom of the page
				writer.Write(TemplateBottom());
			}
			catch(Exception ex)
			{
				SetError("Error rendering page (AAD)", ex);
			}
		}
		#endregion
	}
}
