// Copyright (c) 2002-2003 Brian Eshelman, EshelTech Consulting.
// All Rights Reserved.  Developed for Dot Net Design Patterns.
// 
// Permission to use, copy, modify, and distribute this 
// software and its documentation for NON-COMMERCIAL purposes
// and without fee is hereby granted provided that this 
// copyright notice appears in all copies. 
//
// THE AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT 
// THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, 
// INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR 
// NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR ANY 
// DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, 
// MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
// Please send any comments to beshelman@esheltech.com
//

using System;

namespace Factors
{
	/// <summary>
	/// The worlds slowest factor operation.
	/// </summary>
	public class Factorer : TestBed.ITestable
	{
		public Factorer()
		{
		}

		#region ITestable Interface
		/// <summary>
		/// Will be used to generate random numbers
		/// </summary>
		private Random r = new Random();

		public System.Collections.SortedList GetTestParameters()
		{
			System.Collections.SortedList list = new System.Collections.SortedList();
			list.Add("Value", typeof(System.UInt64));
			return list;
		}
		public System.Collections.SortedList[] GetTestSet(int Count)
		{
			System.Collections.SortedList[] alist = new System.Collections.SortedList[Count];

			for (int i=0; i < Count; i++)
			{
				alist[i] = new System.Collections.SortedList();
				alist[i].Add("Value", r.Next(100000));
			}
			return alist;
		}
		public bool Test(ref System.Collections.SortedList Parameters)
		{
			System.UInt64 val = System.UInt64.Parse(Parameters["Value"].ToString());
			Parameters.Add("Result", Factorer.Factor(val));
			return true;
		}
		#endregion 

		/// <summary>
		/// Find the all the possible factors for a value
		/// </summary>
		/// <param name="ValueToFactor">The integer value to be factored</param>
		/// <returns>Comma delimeted list of factors</returns>
		public static string Factor(System.UInt64 ValueToFactor)
		{
			if (ValueToFactor <= 0)
				throw new Exception("Only values greater than zero can be factored");

			string strRet = "";
			// The absolutely slowest method I can write
			for (System.UInt64 uIndex = 1; uIndex <= ValueToFactor; uIndex++)
			{
				double dFloatCalc = (double)ValueToFactor / (double)uIndex;
				System.UInt64 uUlongCalc = ValueToFactor / uIndex;
				if (dFloatCalc == (double)uUlongCalc)
				{
					if (strRet.Length == 0)
						strRet = uIndex.ToString();
					else
						strRet += ", " + uIndex.ToString();
				}
			}

			return strRet;
		}

	}
}
