using System;
using System.Collections;
using System.Web.Caching;

namespace LazyCaching
{
	/// <summary>
	/// Wraps the existing cache object container used to hold all cached objects, this can be Http Cache or
	/// any custom cache object
	/// </summary>
	public class CacheAdapter : IEnumerable 
	{
		private Cache m_oCacheContainer;
		private IEnumerable m_oCacheEnum;

		/// <summary>
		/// Main ctor, takes most cache container e.g. Http Cache object
		/// </summary>
		/// <param name="oCacheContainer"></param>
		public CacheAdapter(object oCacheContainer)
		{
			// set generic container object this will be cast for specific cache functionality
			if (oCacheContainer is Cache) CacheContainer = (Cache)oCacheContainer;
			// set specific properties that all cached objects should implement
			CacheEnum = (IEnumerable)oCacheContainer;			
		}

		/// <summary>
		/// Property for the main cache, should be edited is cached container type is changed.
		/// </summary>
		private Cache CacheContainer
		{
			get {return m_oCacheContainer;}
			set {m_oCacheContainer = value;}
		}

		/// <summary>
		/// Property for the enumerable interface which any cache object should implement
		/// </summary>
		private IEnumerable CacheEnum
		{
			get {return m_oCacheEnum;}
			set {m_oCacheEnum = value;}
		}

		/// <summary>
		/// Delegate to cache object's enumerator object to support for/each, etc..
		/// </summary>
		public IEnumerator GetEnumerator()
		{
			return CacheEnum.GetEnumerator();
		}

		/// <summary>
		/// Delegates to cache's count property
		/// </summary>
		public int Count 
		{	
			get {return CacheContainer.Count;}
		}

		/// <summary>
		/// Delegates to cache's item property
		/// </summary>
		public object this[string sKey]
		{
			get {return CacheContainer[sKey];} 
			set {CacheContainer[sKey] = value;}
		}

		/// <summary>
		/// Just delegates to the item property (indexer)
		/// </summary>
		/// <param name="sKey">see Cache object</param>
		/// <returns>see Cache object</returns>
		public object Get(string sKey)
		{
			return this[sKey];
		}

		/// <summary>
		/// Just delegates to the insert method of the cache, most cache container should implement a similar object
		/// </summary>
		/// <param name="sKey">see Cache object</param>
		/// <param name="oValue">see Cache object</param>
		/// <param name="oAbsoluteExpiration">see Cache object</param>
		/// <param name="oSlidingExpiration">see Cache object</param>
		public void Insert(string sKey, object oValue, DateTime oAbsoluteExpiration, TimeSpan oSlidingExpiration)
		{
			CacheContainer.Insert(sKey, oValue, null, oAbsoluteExpiration, oSlidingExpiration);
		}

	}
}
