using System;
using System.Diagnostics;
using System.Collections;

namespace DotNetPatterns.Common
{

	/// <summary>
	/// Provides methods to instrument (timing) specific code areas.  To begin, enstantiate the 
	/// class passing the options of using the Eventlog and at what rate to write those changes (flushrate).
	/// To instrument a block of code call BeginInstrumentation() and at the end of that code call
	/// EndInstrumentation() to take a snapshot and write the result.
	/// </summary>
	/// <remarks>
	/// 1. This code has not yet been designed to minimize performance impedence during instrumentation.
	/// </remarks>
	public class LazyInstrumentation
	{
		/// <summary>
		/// 
		/// </summary>
		private long m_lStartTime;
		
		/// <summary>
		/// 
		/// </summary>
		private ArrayList m_alTargetMessages = null;
		
		/// <summary>
		/// 
		/// </summary>
		private int m_nFlushRate; 

		/// <summary>
		/// 
		/// </summary>
		public LazyInstrumentation() : this(true, 10)
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="bUseEventLog"></param>
		/// <param name="nFlushRate"></param>
		public LazyInstrumentation(bool bUseEventLog, int nFlushRate)
		{
			//Trace.Listeners.Add(new EventLogTraceListener((EventLog)(EventLog.GetEventLogs()).GetValue(0)));
			//if (bUseEventLog && Trace.Listeners[Constants.PAYMETRIC_TRACE_EVENTLOG_LISTENER_KEY] == null)
			//	Trace.Listeners.Add(new EventLogTraceListener(Constants.PAYMETRIC_TRACE_EVENTLOG_LISTENER_KEY));
			//TargetMessages = new ArrayList();
			//FlushRate = nFlushRate;
		}

		/// <summary>
		/// 
		/// </summary>
		public int FlushRate
		{
			get { return m_nFlushRate; }
			set { m_nFlushRate = value; }
		}

		/// <summary>
		/// 
		/// </summary>
		public ArrayList TargetMessages
		{
			get { return m_alTargetMessages; }
			set { m_alTargetMessages = value; }
		}

		/// <summary>
		/// 
		/// </summary>
		[Conditional("DEBUG")]
		public void BeginInstrumentation()
		{
			m_lStartTime = (DateTime.Now.Ticks/10000);
		}

		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="sTarget"></param>
		/// <param name="bFlush"></param>
		[Conditional("DEBUG")]
		public void EndInstrumentation(string sTarget, bool bFlush)
		{	
			string sMessage = null;

			sMessage = sTarget + " - " + Convert.ToString((DateTime.Now.Ticks/10000) - m_lStartTime) + "ms";
			if (bFlush || FlushRate <= 1)
			{
				//Trace.WriteLineIf(Config.TraceVerbose, sMessage);
			}
			else
			{
				TargetMessages.Add(sMessage);	
                if (TargetMessages.Count >= FlushRate)
					Flush();
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public void Flush()
		{
			string sFullMessage = "Flushing Data....(instrumentation details)\n------------------------------\n";
	
			foreach (object sMessage in TargetMessages)
			{
				sFullMessage = sFullMessage + (string)sMessage + "\n";
			}
			
			Trace.WriteLine(sFullMessage);
			TargetMessages = new ArrayList();
			
		}

	}

}

