using System;
using System.Diagnostics;
using System.Net;

namespace Common
{
	/// <summary>
	/// This represents the remote tracing custom trace listener that will be added during initialization
	/// and allow all tracing to be sent remotely, the tracing level is determined by the trace switch which is
	/// accessable via the Config.TraceLevel static data member, to send remote traces (if turned on), the clent
	/// must use the following code template:
	/// 
	///		Trace.WriteLineIf(Config.TraceLevel.TraceVerbose, "starting packet translation...");
	///		
	/// </summary>
	/// <example>Trace.WriteLineIf(Config.TraceLevel.TraceVerbose, "starting packet translation...");</example> 
	/// <remarks>
	/// To change the trace level you must edit the launching applications configuration file as such:
	/// The name attribute must match the name given to the trace extension, e.g. RemoteTraceLevel
	/// <system.diagnostics>
	///		<switches>
	///			<add name="RemoteTraceLevel" value="1" />
	///		</switches>
	/// </system.diagnostics>
	/// 0 (off), 1 (error), 2 (warning), 3 (info), OR 4 (verbose)
	/// </remarks>
	public class RemoteTraceListener : System.Diagnostics.TraceListener 
	{
		/// <summary>
		/// 
		/// </summary>
		private localhost.RemoteTracerService oRemoteService = null;

		
		/// <summary>
		/// 
		/// </summary>
		public localhost.RemoteTracerService RemoteService
		{
			get { return oRemoteService; }
			set { oRemoteService = value; }
		}

		/// <summary>
		/// 
		/// </summary>
		public RemoteTraceListener(string sName)
		{
			// initializes the remote web service that will receive the remote traces
			RemoteService = new localhost.RemoteTracerService();
			base.Name = sName;
		}
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="sMessage"></param>
		public override void Write(string sMessage)
		{
			RemoteService.RemoteTrace(Dns.GetHostName(), sMessage);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="sMessage"></param>
		public override void WriteLine(string sMessage)
		{
			RemoteService.RemoteTrace(Dns.GetHostName(), sMessage);
		}

	}
}

