using System;
using System.Text;
using System.Data;
using System.Diagnostics;
using DotNetPatterns.Common;

namespace DataTierPatterns.Data
{
	public class ModelFactory
	{

		public ModelFactory()
		{
		}

		static public PolyModelData CreatePolyModelData()
		{
			if(GetProviderType() == DTPConstants.DBTYPE.SQL)
				return new MSSQLPolyModelData();

			return null;
		}

		static public PolyModelData CreatePolyModelData(string sSQL)
		{
			if(GetProviderType() == DTPConstants.DBTYPE.SQL)
				return new MSSQLPolyModelData(sSQL);

			return null;
		}

		static public PolyModelData CreatePolyModelData(StringBuilder sbSQL)
		{
			if(GetProviderType() == DTPConstants.DBTYPE.SQL)
				return new MSSQLPolyModelData(sbSQL);

			return null;
		}

		
		/// <summary>
		/// This helper method is responsible for creating the correct derived PolyModelData class for the ADO.NET Native or OLEDB Provider
		/// that is to be used for all RDBMS I/O.  This method will instantiate the correct PolyModelData derived object and call the 
		/// appropriate constructor.
		/// </summary>
		/// <param name="sSQL">SQL command to execute.</param>
		/// <param name="eType">IDbCommand CommandType</param>
		/// <returns>An instance of PolyModelData that reflects the ADO.NET Native or OLEDB Provider that is to be used for all RDBMS I/O.</returns>
		static public PolyModelData CreatePolyModelData(string sSQL, CommandType eType)
		{
			if(GetProviderType() == DTPConstants.DBTYPE.SQL)
				return new MSSQLPolyModelData(sSQL, eType);

			return null;
		}

		
		/// <summary>
		/// This helper method is responsible for creating the correct derived PolyModelData class for the ADO.NET Native or OLEDB Provider
		/// that is to be used for all RDBMS I/O.  This method will instantiate the correct PolyModelData derived object and call the 
		/// appropriate constructor.
		/// </summary>
		/// <param name="sbSQL">SQL command to execute.</param>
		/// <param name="eType">IDbCommand CommandType</param>
		/// <returns>An instance of PolyModelData that reflects the ADO.NET Native or OLEDB Provider that is to be used for all RDBMS I/O.</returns>
		static public PolyModelData CreatePolyModelData(StringBuilder sbSQL, CommandType eType)
		{
			if(GetProviderType() == DTPConstants.DBTYPE.SQL)
				return new MSSQLPolyModelData(sbSQL, eType);

			return null;
		}

		/// <summary>
		/// This helper method is responsible for creating the correct derived CommonData class for the ADO.NET Native or 
		/// OLEDB Provider that is to be used for all business logic oriented RDBMS I/O.  
		/// </summary>
		/// <returns>An instance of CommonData that reflects the ADO.NET Native or OLEDB Provider that is to be used for allall business logic oriented RDBMS I/O.</returns>
		static public PolyModel CreatePolyModel()
		{
			// can be used as a factory later when other provider types are added -- e.g. Data Factory
			if(GetProviderType() == DTPConstants.DBTYPE.SQL)
				return new MSSQLPolyModel();

			return null;
		}		

		static public PolyModelWhere CreatePolyModelWhere()
		{
			if(GetProviderType() == DTPConstants.DBTYPE.SQL)
				return new MSSQLPolyModelWhere();

			return null;
		}		

		static public PolyModelWhereCollection CreatePolyModelWhereCollection()
		{
			if(GetProviderType() == DTPConstants.DBTYPE.SQL)
				return new MSSQLPolyModelWhereCollection();

			return null;
		}		

		static public DTPConstants.DBTYPE GetProviderType()
		{
			try
			{
				string sValue = string.Empty;
				sValue = Config.GetAppSetting("DBType");
				
				if(sValue == "MSSQL")
					return DTPConstants.DBTYPE.SQL;				
			}
			catch(Exception e)
			{
				Trace.WriteLineIf(Config.TraceError, e.Message);
			}

			return DTPConstants.DBTYPE.UNKNOWN;
		}		

	}
}
