using System;
using System.Collections;
using DataTierPatterns.Data;

namespace DataTierPatterns.Data
{
	///	SchemaPolyModelWhereCollection is an abstract class that manages a collection of PMWhere objects.  In order to effectivly use this 
	///	class, a new class must be derives for each .NET and/or OLEDB database provider that needs to be supported.  
	///	All methods that are not abstract are desginated as virtual for those cases where the logic implemented in this	class 
	///	is not sufficient to provide the necessary support needed by the framework.  
	public abstract class PolyModelWhereCollection : System.Collections.CollectionBase
	{	
		/// <summary>
		/// Adds a PMWhere object to the internal collection.  Strongle typed for type safety.
		/// </summary>
		/// <param name="oWhere">PMWhere object to add to collection</param>
		public void Add(PolyModelWhere oWhere)
		{
			List.Add(oWhere);
		}

		/// <summary>
		/// Overloaded Add that will create a PMWhere object JIT just to add a key/value pair to the collection.
		/// </summary>
		/// <param name="sKey">Lookup Key</param>
		/// <param name="sValue">Value to match</param>
		public void Add(string sKey, string sValue)
		{
			PolyModelWhere oWhere = null;
			oWhere = ModelFactory.CreatePolyModelWhere();
			if(oWhere != null)
			{
				oWhere.WhereEqual(sKey, sValue);
				Add(oWhere);
			}
		}

		/// <summary>
		/// Removes a PMWhere object from the internal collection.  Strongle typed for type safety.
		/// </summary>
		/// <param name="index">Index of object to remove</param>
		public void Remove(int index)
		{
			// Check to see if there is a PolyModelWhere at the supplied index.
			if(index > Count - 1 || index < 0)				
			{
				// No PolyModelWhere exists, the operation is cancelled.
			}
			else
			{
				List.RemoveAt(index); 
			}
		}

		/// <summary>
		/// Returns a PMWhere object from the internal collection.   Strongle typed for type safety.
		/// </summary>
		/// <param name="Index">Index of object to return</param>
		/// <returns>PMWHere</returns>
		public PolyModelWhere Item(int Index)
		{
			// The appropriate item is retrieved from the List object and
			// explicitly cast to the PolyModelWhere type, then returned to the 
			// caller.
			return (PolyModelWhere) List[Index];
		}
		
		/// <summary>
		/// This asbtract method must be implemented in all derived classes.  This method is intended to generate a dynamic
		/// SQL command based on the where criteria stored in the internal collection.  The SQL command is specific to the 
		/// underlying Paymetric RDBMS ERD.  Where criteria mapps directly to the DATA_IDX table so ensure that a identitity
		/// schema is setup for each XSD Product Schema.  During database writes of transaction data, the identity schema is
		/// used to populate the DATA_IDX table. 
		/// </summary>
		/// <returns>SQL command to execute</returns>
		public abstract string GetSQL();
		
		/// <summary>
		/// This asbtract method must be implemented in all derived classes.  This method is intended to generate a dynamic
		/// SQL command based on the where criteria stored in the internal collection and the passed in schema ID.
		/// The SQL command is specific to the underlying Paymetric RDBMS ERD.  Where criteria mapps directly to the DATA_IDX 
		/// table so ensure that a identitity schema is setup for each XSD Product Schema.  During database writes of 
		/// transaction data, the identity schema is used to populate the DATA_IDX table.
		/// </summary>
		/// <param name="lSchemaID">XSD Product Schema ID</param>
		/// <returns>SQL command to execute</returns>
		public abstract string GetSQL(long lSchemaID);
	}
}
