using System;
using System.Text;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Runtime.Serialization;
using System.Xml;
using System.Web.Services.Protocols;

namespace ExceptionHandling
{
	/// <summary>
	/// The is the base exception class for Patterns.NET
	/// </summary>
	[Serializable]
	public class BaseException : System.ApplicationException
	{
		private int m_nCode;
		
		public BaseException(){;}

		public BaseException(string sMessage, bool bLog) : this(sMessage, null, bLog){;}
		
		public BaseException(string sMessage, System.Exception oInnerException, bool bLog) : this(null, 0, sMessage, oInnerException, bLog){;}
		
		public BaseException(object oSource, int nCode, string sMessage, bool bLog) : this(oSource, nCode, sMessage, null, bLog){;}
		
		public BaseException(object oSource, int nCode, string sMessage, System.Exception oInnerException, bool bLog) : base(sMessage, oInnerException)
		{
			if (oSource != null)
				base.Source = oSource.ToString();
			Code = nCode;

			if (bLog)
			{
				EventLog.WriteEntry("BaseException", sMessage);
			}
		}
		#region SERIALIZATION CODE
		/*protected BaseException(SerializationInfo oInfo, StreamingContext oContext) : base(oInfo, oContext)
		{
			m_nCode = oInfo.GetInt32("m_nCode");
		}

		public override void GetObjectData(SerializationInfo oInfo, StreamingContext oContext)
		{
			oInfo.AddValue("m_nCode", m_nCode, typeof(int));
			base.GetObjectData(oInfo, oContext);
		}*/
		#endregion

		public int Code 
		{
			get {return m_nCode;}
			set {m_nCode = value;}
		}
		
		/// <summary>
		/// This ctor for throwing soap exception usually from web service methods
		/// This will format error message in more more soap fault friendly manner
		/// filling in fields not filled in by the default wrapper exception method in .NET
		/// </summary>
		/// <param name="oSource"></param>
		/// <param name="nCode"></param>
		/// <param name="sMessage"></param>
		/// <param name="oInnerException"></param>
		/// <param name="bLog"></param>
		/// <param name="bThrowSoap"></param>
		public BaseException(object oSource, int nCode, string sMessage, System.Exception oInnerException, bool bLog, bool bThrowSoap) : this(oSource, nCode, sMessage, oInnerException, bLog)
		{
			string sCause = "no root cause found";
			XmlNode oNode = null;

			if (bThrowSoap)
			{
				if (oInnerException != null)
				sCause = oInnerException.GetBaseException().Message;
					
				// now build the details node to use for the soap exception -- use the root cause for the main text
				oNode = BuildDetailNode(sCause);
					
				Trace.WriteLine("Throwing Custom Soap Exception - Message " + sMessage);
					
				// build actor or source uri to set into actor field -- replace .'s with /'s to make a uri from it
				// NOTE: the web service must namespace match the reqeues uri if this is to be accurate
				// we can't use context without passing it so we'll build one
				StringBuilder sActor = new StringBuilder("http://");
				sActor.Append(Dns.GetHostName());
				sActor.Append("/");
				sActor.Append("dotnetpatterns");
				sActor.Append("/");
				sActor.Append(oSource.ToString().Replace(".", "/"));
				sActor.Append(".asmx");
						
				throw new SoapException(sMessage, SoapException.DetailElementName, sActor.ToString(), oNode, oInnerException);
			}
		}

		/// <summary>
		/// Builde the xml node used for throwing custom soap exception, the root cause string will be use for the main content
		/// </summary>
		/// <param name="sCause"></param>
		/// <returns></returns>
		public System.Xml.XmlNode BuildDetailNode(string sCause)
		{
			XmlDocument oDoc = new XmlDocument();
			XmlNode oNode = oDoc.CreateNode(XmlNodeType.Element, SoapException.DetailElementName.Name, SoapException.DetailElementName.Namespace);

			// Build specific details for the SoapException.
			// Add first child of detail XML element.
			XmlNode oDetailsNode = oDoc.CreateNode(XmlNodeType.Element, "rootcause", "http://www.etier.com/patterns.net");
			oDetailsNode.InnerXml = sCause;
			
			oNode.AppendChild(oDetailsNode);
			return oNode;
		}

	}
}



