#ifndef LCTCLIENT_H_INCLUDED
#define LCTCLIENT_H_INCLUDED

#pragma once

#include <atlsoap.h>


#define		LCTCLIENT_EXECUTION	101

class CLCTClientSoapSocket;
//##ModelId=3CFFC3BB00D7
typedef void (*SOAP_ASYNC_CALLBACK)(LPARAM lParam, HRESULT hRet);

//##ModelId=3CFFC3BB00F4
struct	stLCTClientRequest
{
	//##ModelId=3CFFC3BB0124
	CLCTClientSoapSocket	*pClient;
	//##ModelId=3CFFC3BB0133
	SOAP_ASYNC_CALLBACK		pfnCallback;
	//##ModelId=3CFFC3BB0142
	LPARAM					lpRequestParam;
};



//##ModelId=3CFFC3BB0152
class CLCTClientWorker
{
public:
	//##ModelId=3CFFC3BB0192
	typedef stLCTClientRequest* RequestType;
	
	//##ModelId=3CFFC3BB0161
	CLCTClientWorker() throw()
	{
	}

	//##ModelId=3CFFC3BB0171
	~CLCTClientWorker() throw()
	{
	}

	//##ModelId=3CFFC3BB0172
	virtual BOOL Initialize(void *pvParam) throw()
	{
		// Do any initialization here
		return TRUE;
	}

	//##ModelId=3CFFC3BB0175
	virtual void Terminate(void* pvParam) throw()
	{
		// Do any cleanup here
	}

	//##ModelId=3CFFC3BB0183
	void Execute(stLCTClientRequest	*pRequestInfo, void *pvParam, OVERLAPPED *pOverlapped) throw();

};


//##ModelId=3CFFC3BB01A1
typedef CThreadPool<CLCTClientWorker>	CLCTClientThreadPool;


//##ModelId=3CFFC3BB01CF
class CLCTClientSoapSocket : public CSoapSocketClientT<>
{
protected:
	//##ModelId=3CFFC3BB020F
	stLCTClientRequest		m_stRequestInfo;
	//##ModelId=3CFFC3BB0220
	bool					m_bInRequest;
	
	//##ModelId=3CFFC3BB022C
	HRESULT					m_hRealRet;
	//##ModelId=3CFFC3BB023D
	CLCTClientThreadPool	*m_pThreadPool;
	//##ModelId=3CFFC3BB020D
	CString					m_strSOAPAction;
	

public:
	//##ModelId=3CFFC3BB024C
	CLCTClientSoapSocket(LPCTSTR szUrl) : CSoapSocketClientT<>(szUrl)
	{
		m_bInRequest = false;
		m_pThreadPool = NULL;
	}


	// Note : this is not thread safe, the caller should take care
	//##ModelId=3CFFC3BB025B
	HRESULT	PrepareRequest(CLCTClientThreadPool	*pThreadPool, SOAP_ASYNC_CALLBACK	pfnCallback, LPARAM lParam)
	{
		// return false, if already in request
		if( m_bInRequest )
			return E_PENDING;
		if( !pThreadPool )
			return E_POINTER;

		m_pThreadPool = pThreadPool;

		m_stRequestInfo.pClient = this;	
		m_stRequestInfo.lpRequestParam= lParam;
		m_stRequestInfo.pfnCallback=pfnCallback;
	
		m_bInRequest = true;

		return S_OK;
	}

	//##ModelId=3CFFC3BB026D
	HRESULT SendRequest(LPCTSTR szAction)
	{
		HRESULT		hRet;

		if( m_bInRequest )
		{
			// launch the async request
			m_strSOAPAction = szAction;
			ATLASSERT(m_pThreadPool);
			if( m_pThreadPool->QueueRequest(&m_stRequestInfo) )
				hRet = E_PENDING;
			else
				hRet = E_FAIL;
		}
		else
		{
			// post-async request, reading the response. 
			// Nothing to do, just return hRet from the reall operation
			hRet = m_hRealRet;
		}

		return hRet;
	}

	//##ModelId=3CFFC3BB027C
	HRESULT	InternalSendRequest()
	{
		m_hRealRet = CSoapSocketClientT<>::SendRequest( m_strSOAPAction );
		
		m_bInRequest = false;
		return m_hRealRet;
	}
};







//##ModelId=3CFFC3BB0183
inline void CLCTClientWorker::Execute(stLCTClientRequest	*pRequestInfo, void *pvParam, OVERLAPPED *pOverlapped) throw()
{
	ATLASSERT(pRequestInfo != NULL);
	pvParam;		// unused
	pOverlapped;    // unused

	CLCTClientSoapSocket	*pClient = pRequestInfo->pClient;
	SOAP_ASYNC_CALLBACK		pfnCallBack = pRequestInfo->pfnCallback;
	LPARAM lParam = pRequestInfo->lpRequestParam;
	
	ATLASSERT( pClient );
	HRESULT hRet = pClient->InternalSendRequest();

	if( pfnCallBack )
	{
		pfnCallBack( lParam, hRet);
	}
}


#endif LCTCLIENT_H_INCLUDED
