using System;
using System.Data;

namespace LCTServer
{
	/// <summary>
	/// Contains those methods required to perform loosely coupled transactions
	/// </summary>
	public interface ITransactor
	{
		/// <summary>
		/// Calls the appropriate stored proc to increment a db count and return it
		/// </summary>
		/// <returns></returns>
		long GenerateTransactionId();
		
		/// <summary>
		/// This creates a new transaction 
		/// as part of a transaction table (uses the specified transactor id)
		/// </summary>
		/// <param name="lTransactorId"></param>
		/// <param name="nStatus"></param>
		/// <returns></returns>
		DataSet CreateTransaction(long lTransactorId, int nStatus);
		
		/// <summary>
		/// Updates the transaction with the dataset results 
		/// of any returned external call
		/// </summary>
		/// <param name="lTransactorId"></param>
		/// <param name="nStatus"></param>
		/// <returns></returns>
		DataSet UpdateTransaction(long lTransactorId, int nStatus, DataSet dsOriginal);
		
		/// <summary>
		/// Returns the transaction as a dataset 
		/// for the transactor id 
		/// </summary>
		/// <param name="lTransactorId"></param>
		/// <returns></returns>
		DataSet GetTransaction(long lTransactorId);

		/// <summary>
		/// This will regenerate the original DataSet from the stored
		/// schema and data as xml in the job table
		/// </summary>
		/// <param name="dsJob"></param>
		/// <returns></returns>
		DataSet ExtractOriginalData(DataSet dsJob);
		
		/// <summary>
		/// Returns the status of the transaction
		/// </summary>
		/// <param name="lTransactorId"></param>
		/// <returns></returns>
		int GetStatus(long lTransactorId);
	}
}
