using System;
using System.Collections;
using System.Threading;
using System.Diagnostics;
using System.Reflection;

namespace ThreadDelegate
{
	/// <summary>
	/// Summary description for EventSource.
	/// </summary>
	public class EventSource
	{
		private Hashtable hshEvents = new Hashtable();
		private Timer	oTimer = null;
		private int nIter=0;
		private Process thisProcess=Process.GetCurrentProcess();

		public EventSource()
		{
		}

		public event EventHandler Event1
		{
			add
			{
				hshEvents["Event1"] = (EventHandler)hshEvents["Event1"] + value;
			}
			remove
			{
				hshEvents["Event1"] = (EventHandler)hshEvents["Event1"] - value;
			}
		}

		public event EventHandler Event2
		{
			add
			{
				hshEvents["Event2"] = (EventHandler)hshEvents["Event2"] + value;
			}
			remove
			{
				hshEvents["Event2"] = (EventHandler)hshEvents["Event2"] - value;
			}
		}

		public void Run(int Iterations)
		{
			nIter = Iterations;
			ManualResetEvent evt = new ManualResetEvent(false);
			oTimer = new Timer(new TimerCallback(this.OnTimedEvent), evt, 0, 5000); 
			evt.WaitOne();
		}

		public void ExecThreadDelegate(object objDlg)
		{
			Delegate dlg = (Delegate)objDlg;

			object[] delegateArgs = new object[] { this, null };

			try
			{
				dlg.DynamicInvoke(delegateArgs);
			}
			catch(TargetInvocationException )
			{
				Console.WriteLine("Delegate {0} threw an exception, removing it from list", dlg.Method.Name);
				Event1 -= (EventHandler)dlg;
			}
		}

		// Specify what you want to happen when the Elapsed event is raised.
		public void OnTimedEvent(object state)
		{
			nIter--;
			if (nIter < 0)
			{
				oTimer.Dispose();
				oTimer = null;
				((ManualResetEvent)state).Set();
				return;
			}

			Console.WriteLine("\nEventSource::OnTimedEvent- Thread Id={0}", Win32Native.GetCurrentThreadId());
			
				// Invoke delegate list for Event1 on the existing thread
			EventHandler evt = (EventHandler)hshEvents["Event1"];
			if (evt != null)
			{
				Console.WriteLine("Invoking Event1 on thread {0}", Win32Native.GetCurrentThreadId());
				object[] delegateArgs = new object[] { this, null };

				HighFrequencyTimer hfTimer = new HighFrequencyTimer();
				hfTimer.Start();

				foreach (Delegate dlg in evt.GetInvocationList())
				{
					try
					{
						dlg.DynamicInvoke(delegateArgs);
					}
					catch(TargetInvocationException )
					{
						Console.WriteLine("Delegate {0} threw an exception, removing it from list", dlg.Method.Name);
						Event1 -= (EventHandler)dlg;
					}
				}
				hfTimer.Stop();
				Console.WriteLine("Event1 delegate took {0:F5} ms to fire.", hfTimer.ElapsedMilliseconds);
			}

			evt = (EventHandler)hshEvents["Event2"];
			if (evt != null)
			{
				Console.WriteLine("\n\t\tQueueing Event2 delgates to ThreadPool");
				HighFrequencyTimer hfTimer = new HighFrequencyTimer();
				hfTimer.Start();

				foreach (Delegate dlg in evt.GetInvocationList())
				{
					ThreadPool.QueueUserWorkItem(new WaitCallback(ExecThreadDelegate), dlg);					
				}

				hfTimer.Stop();
				Console.WriteLine("\t\tEvent2 delegate took {0:F5} ms to fire.", hfTimer.ElapsedMilliseconds);
			}

			thisProcess.Refresh();

		}

	}
}
